<?php
// $Horde: mnemo/lib/Mnemo.php,v 1.2.2.1 2002/05/31 16:08:13 jan Exp $

require_once MNEMO_BASE . '/lib/constants.php';

/**
 * Mnemo Base Class.
 *
 * @author Jon Parise <jon@horde.org>
 * @version $Revision: 1.2.2.1 $
 * @package mnemo
 */
class Mnemo {

    /**
     * Retrieves the current user's memo list from storage.
     * This function will also sort the resulting list, if requested.
     *
     * @param object Mnemo_Driver  $storage      The current storage object.
     * @param constant $criteria (optional) A set of flags specifying the set of
     *                           memos to return. (MEMO_ANY, MEMO_ADDED,
     *                           MEMO_MODIFIED, MEMO_DELETED)
     * @param constant $sortby   (optional) The field by which to sort.
     *                           (MNEMO_SORT_DESC, MNEMO_SORT_CATEGORY)
     * @param constant $sortdir  (optional) The direction by which to sort.
     *                           (MNEMO_SORT_ASC, MNEMO_SORT_DESC)
     *
     * @return array        Returns a list of the requested memos.
     *
     * @see Mnemo_Driver::listMemos()
     */
    function listMemos($storage, $criteria = MEMO_ANY, $sortby = MNEMO_SORT_DESC,
                       $sortdir = MNEMO_SORT_ASCEND)
    {
        /* Retrieve the memo list for storage. */
        $memos = $storage->listMemos($criteria);

        /* Sort the memo list. */
        $sort_functions = array(
            MNEMO_SORT_DESC => 'ByDesc',
            MNEMO_SORT_CATEGORY => 'ByCategory');

        /* Sort the array if we have a sort function defined for this field. */
        if (isset($sort_functions[$sortby])) {
            $prefix = ($sortdir == MNEMO_SORT_DESCEND) ? '_rsort' : '_sort';
            uasort($memos, array('Mnemo', $prefix . $sort_functions[$sortby]));
        }

        return $memos;
    }

    /**
     * Builds the HTML for a memo category widget.
     *
     * @param string  $name       The name of the widget.
     * @param integer $selected   (optional) The default category.
     *
     * @return string       The HTML <select> widget.
     */
    function buildCategoryWidget($name, $selected = 0)
    {
        $html = "<select name=\"$name\">";

        foreach (Mnemo::listCategories() as $id => $name) {
            $html .= "<option value=\"$id\"";
            $html .= ($id == $selected) ? ' selected="selected">' : '>';
            $html .= $name . '</option>';
        }
        $html .= '</select>';

        return $html;
    }

    /**
     * List a user's categories
     *
     * @return array A list of categories.
     */
    function listCategories()
    {
        global $prefs;

        static $catString, $categories;

        if (!isset($categories) || $catString != $prefs->getValue('memo_categories')) {
            $catString = $prefs->getValue('memo_categories');
            $cats = explode('|', $catString);
            $categories = array(0 => _("Unfiled"));
            foreach ($cats as $cat) {
                list($key, $val) = explode(':', $cat);
                $categories[$key] = $val;
            }
        }

        return $categories;
    }

    /**
     * Returns the string matching the given category ID.
     *
     * @param integer $categoryID     The category ID to look up.
     *
     * @return string       The formatted category string.
     */
    function formatCategory($categoryID = 0)
    {
        $categories = Mnemo::listCategories();
        return isset($categories[$categoryID]) ?
            $categories[$categoryID] :
            $categories[0];
    }

    /**
     * Comparison function for sorting memos by description.
     *
     * @param array $a  Memo one.
     * @param array $b  Memo two.
     *
     * @return integer  1 if memo one is greater, -1 if memo two is greater; 0 if they are equal.
     */
    function _sortByDesc($a, $b)
    {
        return strcmp($a['desc'], $b['desc']);
    }

    /**
     * Comparison function for reverse sorting memos by description.
     *
     * @param array $a  Memo one.
     * @param array $b  Memo two.
     *
     * @return integer  -1 if memo one is greater, 1 if memo two is greater; 0 if they are equal.
     */
    function _rsortByDesc($a, $b)
    {
        return strcmp($b['desc'], $a['desc']);
    }

    /**
     * Comparison function for sorting memos by category.
     *
     * @param array $a  Memo one.
     * @param array $b  Memo two.
     *
     * @return integer  1 if memo one is greater, -1 if memo two is greater; 0 if they are equal.
     */
    function _sortByCategory($a, $b)
    {
        return strcmp($a['category'], $b['category']);
    }

    /**
     * Comparison function for reverse sorting memos by category.
     *
     * @param array $a  Memo one.
     * @param array $b  Memo two.
     *
     * @return integer  -1 if memo one is greater, 1 if memo two is greater; 0 if they are equal.
     */
    function _rsortByCategory($a, $b)
    {
        return strcmp($b['category'], $a['category']);
    }

}
?>
